--Here: Test-harness main program to test module CTrees (CTrees.hs).
--Compile tests thus: ghc --make CTrees
--has run under ghc 6.4.1, Feb 2008).

module Main where

import IP.Utilities
import IP.Classes
import IP.Models
import IP.CTrees     -- *

main = let
 { ct01_leaf1 = CTleaf (freqs2model [2,2]) ;  -- 1:1
   ct01_leaf2 = CTleaf (freqs2model [1,3]) ;  -- 3:7
   ct01 = CTforkFM
            (FM 0 (\ip->MPr 0 (\op->if ip==op then 1.0 else 0.0) (\()->"?"))
                  (\()->"?"))
            [ct01_leaf1, ct01_leaf2] ;
   -- NB. ct01 (trivially) has a FunctionModel in the fork nodes

   ct02_leaf0 = CTleaf (coinByPrH 0.5);
   ct02_leaf1 = CTleaf (coinByPrH 0.2);
   ct02 = CTfork 0 (Splitter 2 (\ip->ip) (\()->"id|{0,1}"))
                 [ct02_leaf0, ct02_leaf1];

   ct03 = estCTree estMultiState splits ips ops ;              -- infer a CTree
   -- NB. above uses default splits for ips; see Utilities.
   ipCases = [ (H, H,  1.0), (H, H,  2.0), (H, T,  1.0), (H, T,  2.0 :: Float),
               (T, H,  1.0), (T, H,  2.0), (T, T,  1.0), (T, T,  2.0),
               (H, H, -1.0), (H, H, -2.0), (H, T, -1.0), (H, T, -2.0),
               (T, H, -1.0), (T, H, -2.0), (T, T, -1.0), (T, T, -2.0) ] ;
   opCases = [ H, H, H, H,   T, T, T, T,
               H, H, H, H,   H, H, H, H ] ;   -- H iff @0 == H or @2 < 1.0
   ips = (take 100 (cycle ipCases)) ++ ipCases ;        --  ++ noise
   ops = (take 100 (cycle opCases)) ++ [T,T,T,T, H,H,H,H, T,T,T,T, T,T,T,T] ;

                                                    -- Normal (Gaussian) leaves
   ct04 = estCTree (estNormal (-10.0) 10.0 0.1 10.0 0.1) splits
                   (take 100 (cycle b2)) (take 100 (cycle cts)) ;
                                                  -- play with sep'n and with n
   -- @0 => N(0,1) v. N(sep', 1);  @1 irrelevant
   b2 = [(True, True), (True, False), (True, True), (True, False),
         (False,True), (False,False), (False,True), (False,False)] ;
   cts = let c = [1.0, 1.0, -1.0, -1.0]
             cplus separation = map (+separation) c
         in c ++ cplus 0.9
 }

 in putStrLn("-- Test of Classification Trees (CTrees) for "
       ++ "Inductive Programming, LA --\n")

 >> putStrLn("condPr ct01 0 0 and 1 0 = "
          ++ show( zipWith (condPr ct01) [0,1] [0,0]) )
 >> putStrLn("condPr ct02 0 H and 1 H = "
          ++ show( zipWith (condPr ct02) [0,1] [H,H] ) )
 >> putStrLn("splits ips = " ++ show( splits ips ) )

 >> putStrLn("\nct03 = " ++ show(ct03) ++ " = "
          ++ show(msg1 ct03) ++ " nits, data|ct03 = "
          ++ show( msg2 (functionModel2model ct03) (zip ips ops) ) ++ " nits")
 >> putStrLn("condPr ct03 [(H,H,1.0) ... ] H = "
          ++ show( zipWith (condPr ct03) ipCases (take 16 (repeat H)) ) )

 >> putStrLn("\nct04 = " ++ show ct04 )

 >> putStrLn "\n-- done --"
-- ----------------------------------------------------------------------------
