--Here: Test-harness main program to test module FnModels (FnModels.hs).
--Compile tests thus: ghc --make FnModels
--(has run under ghc 6.4.1, April 2008).

module Main where

import IP.Utilities
import IP.Classes
import IP.Models
import IP.FnModels   -- *

-- NB. tuples of Enum Bounded types are made instances of Enum in Utilities.


main = let
 { coin1 = [ H, H, H, H, T, T, T, T];     -- inputs
   coin2 = [ H, H, H, T, T, T, T, H];     -- outputs
   fm01 = estFiniteFunction coin1 coin2;  -- 3.5:1.5 = 7:3 same:different
   cndn = estCondition estMultiState estFiniteFunction (zip coin1 coin2); --2|1

   -- NB. tuples of Enum Bounded types are made instances of Enum in Utilities
   bool2  = [(True,True), (True,False), (False,True), (False,False)];
   bool2X = [ False,       True,         True,         False       ];
   xor = estFiniteFunction (take 12 (cycle bool2))  --  add some noise
                           ((take 8 (cycle bool2X)) ++ (map not bool2X));

   inpts = [[H,H],[H,T],[T,H],[T,T], [H,T],[T,H]];  -- inputs of length 2
   rslts = [  H,    T,    H,    T,     T,    T  ];  -- results
   fm02 = estFiniteListFunction 2 inpts rslts;      -- 3:1 1:5 1:1 1:3

   l0    = linear1 0.1 2.0 1.0 1.0;                 -- fixed N(2x+1, 1.0)
   l1ys  = [-3.5, -0.5, 0.5, 3.5, 22.5, 25.5];      -- y ~ 2x+1 +/-0.5
   l1xs  = [-2.0, -1.0, 0.0, 1.0, 11.0, 12.0];      --
   testLinear1 xs ys =
     let l1    = estLinear1 (-10) 10 0.01 100.0 0.1 xs ys
         l1xys = zip xs ys
     in putStrLn("linear1 (" ++ show(length xs) ++ " data): " ++ show l1
          ++ ", msg1=" ++ show(msg1 l1)
          ++ ", msg2=" ++ show(msg2 (functionModel2model l1) l1xys) ++ " nits")
 }

 in putStrLn "-- Tests for FnModels in Inductive Programming, LA --\n"

 >> putStrLn("fm01 = " ++ show fm01 )
 >> putStrLn("fm01 H H,... H T  = "
     ++ show( zipWith (condPr fm01) [H,H,T,T] [H,T,H,T] ))
 >> putStrLn("cndn " ++ show cndn)
 >> putStrLn("noisy xor =         " ++ show xor )
 >> putStrLn("fm02 = " ++ show fm02 )
 >> putStrLn("fm02 HH ... -> H  = "
     ++ show( zipWith (condPr fm02) [[H,H],[H,T],[T,H],[T,T]] [H,H,H,H]))
 >> putStrLn("linear1, l0: "++ show l0
          ++ ", pr(1|0)=" ++ show(condPr l0 0.0 1.0))
 >> putStrLn ""
 >> testLinear1 l1xs l1ys
 >> testLinear1 (take 60 (cycle l1xs)) (take 60 (cycle l1ys))
 >> putStrLn ""
 >> testLinear1 [] []                  -- pathological data
 >> testLinear1 [0.0] [1.0]            --   "
 >> testLinear1 [0.0, 1.0] [1.0, 2.0]  --   "

 >> putStrLn "\n-- done --"
-- -------------------------------------------LA--http://www.allisons.org/ll/--
