--Here: Test-harness main program to test module MixtureModels
--      (MixtureModels.hs).
--Compile tests thus: ghc --make MixtureModels
--(has run under ghc 6.4.1, April 2008).

module Main where

import IP.Utilities
import IP.Classes
import IP.Models
import qualified IP.T as T  -- Student's t-distribution (also see Models)

import IP.MixtureModels   -- *

main = let
  twoUp = (take 100 . cycle)                       -- (imaginary) 2-up data ...
          [(H,H),(H,H),(H,H),(H,H), (T,T),(T,T),(T,T),(T,T), (H,T),(T,H)]
          --you'll notice that the 2 coins must be psychic although
          --individually each coin seems to be unbiased.
  coins2 = [(H,H), (H,T), (T,H), (T,T)]                           -- fair coins

  est2coins = estBivariateWeighted ( estMultiStateWeighted,
                                     estMultiStateWeighted )
  de1 = estMixture [est2coins]                       -- 1:1:1:1
  de2 = estMixture [est2coins, est2coins]            -- 4:1:1:4 +/-
  de3 = estMixture [est2coins, est2coins, est2coins]

  testDiscEst str est = testDisc str (est twoUp)

  testDisc str mm =
    let fair_coins = (take 100 . cycle) coins2
    in putStrLn("Model is " ++ str ++ " = " ++ show mm)
    >> putStrLn("  pr  "++str++" [HH HT TH TT] = " ++ show(map (pr mm) coins2))
    >> putStrLn("  msg("++str++" & twoUp_data) = " ++ show( msg mm twoUp )
        ++ " = " ++ show(msg1 mm) ++ "+" ++ show(msg2 mm twoUp) ++ " nits")
    >> putStrLn("  msg("++str++" & fair_coins) = "
        ++ show(msg mm fair_coins) ++ " nits = "
        ++ show(msgBase 2 mm fair_coins) ++ " bits\n")

  -- --------------------------------------------------------------------------
                                                  -- univariate continuous data
  n   = 120  --play around with n and sep
  sep = 4.0  --separation between the means of the 2 classes
  f1 = take (2*n `div` 3) (cycle [-1.0, 1.0, 0.9, -0.9, 1.1, -1.1])  -- ~N(0,1)
  f2 = take   (n `div` 3) (cycle (map ((+) sep) f1))               -- ~N(sep,1)
  -- BEWARE: Initially I had |f1|==|f2| and the (simple minded) search had
  --         a hard time getting out of a rather stable start state
  --         the search algorithm could be improved.
  flts = f1 ++ f2                                -- training data set
  en = estNormalWeighted (-10) 10 0.1 10.0 0.1   -- Normal estimator

  norm1 = en f1 (repeat 1)
  norm2 = en f2 (repeat 1)
  cm0 = mixture (Mix (probs2model [2/3,1/3]) [norm1, norm2])     -- ~true model
  ce1 = estMixture [en]                                       --gives one class
  ce2 = estMixture [en, en]                                 --gives two classes

  et = T.est_wtd (-10) 10 0.1 10.0 0.1      --est. for Student's t-distribution
  te1 = estMixture [et]
  te2 = estMixture [et, et]

  testCtsEst str est dataset = testCtsMdl str (est dataset) dataset

  testCtsMdl str mm dataset =
    putStrLn("Model is " ++ str ++ " = " ++ show mm)
    >> putStrLn("  pr "++str++" [-1 0 1]    = "
             ++ show(map (pr mm) [-1.0, 0.0, 1.0]))
    >> putStrLn("  msg("++str++" & dataset) = "
        ++ show(msg mm dataset) ++ " = "
        ++ show(msg1 mm) ++ "+" ++ show(msg2 mm dataset) ++ " nits\n")

-- ----------------------------------------------------------------------------
 in putStrLn("-- Test of Mixture Models (Clustering) for "
          ++ "Inductive Programming, LA --\n")

 >> putStrLn "Discrete data (bivariate, 2 classes):"
 >> testDiscEst "1-class  " de1    --too simple for twoUp, best for fair coins
 >> testDiscEst "2-classes" de2    --best for twoUp
 >> testDiscEst "3-classes" de3    --overly complex

 >> putStrLn "Continuous data:"
 >> testCtsMdl  "cheat  " cm0 flts
 >> testCtsEst  "1*N(,) " ce1 flts --Normals
 >> testCtsEst  "2*N(,) " ce2 flts
 >> testCtsEst  "1*t(,,)" te1 flts --Student's t-distribution is
 >> testCtsEst  "2*t(,,)" te2 flts --slower than N(,), of course

 >> putStrLn "-- NB. cts data stated to +/-0.05 in most tests."
 >> putStrLn "-- done --"
-- -------------------------------------------LA--http://www.allisons.org/ll/--
