--Here: Test-harness main program to test module Models (Models.hs).
--Compile tests thus: ghc --make Models
--(has run under ghc 6.4.1, Feb 2008).

module Main where

import IP.Utilities
import IP.Classes
import IP.Models     -- *
import qualified IP.T as T

main =
 let multi01 = freqs2model [1.1, 1.1, 2.2]     -- test fractional freq's
     coin01  = coinByPrH 0.5                   -- 50:50
     coin02  = estMultiState [H,H,T]           --  5:3  !
  -- NB. tuples of Enum Bounded types made instances of class Enum in Utilities
     twoCoin = estMultiState [(H,H),(T,T), (H,T),(H,T), (T,H), (T,H) ]
     bitsMdl = freqs2model [100,100]
     ints1   = [6, 7, 9, 10, 11, 15, 20] :: [Int]

     estGeom = estGeometric 10
     estPois = estPoisson   10
     estInorm= estIntNormal 0 30 1 10

     flts1   = [1.0, 2.0, 0.0, 1.0, -1.0, 3.0] -- roughly N(1,1.4)
     flts2   = take 60 (cycle flts1)           -- just more of the same
     flts3   = flts2 ++ [-19.0, 21.0]          -- flts2 with fatter tail
     n01     = normal  0.1  0  1               -- NB. data measured to  +/- 0.1
     estNorm = estNormal (-10.0) 10.0  0.1 10.0  0.1
     estT    = T.est     (-10.0) 10.0  0.1 10.0  0.1

     test_est est ds =
       let m = est ds   -- estimate model m from dataset ds
       in test_model m ds

     test_model m ds =
       let l     = length ds
           few_l = if l <= 10 then l else 5
           more  = if few_l < l then "..." else ""
           few   = take few_l ds
           m_ds  = map (pr m) ds
       in putStrLn("Model is " ++ show m)
       >> putStrLn("  map (pr m) " ++ show few++more ++ " = "
                ++ show (take few_l m_ds)++more
		++ " (sum="++show(sum m_ds)++")")
       >> putStrLn("  map (nlPrBase 2 m) ...    = "
                ++ show(map (nlPrBase 2 m) few)++more++" bits")
       >> putStrLn("  msgLen(" ++ show l ++ " data) = "
                ++ show(msg m ds) ++ " = "
                ++ show(msg1 m) ++ " + " ++ show(msg2 m ds) ++ " nits")

 in putStrLn "-- Test of Models for Inductive Programming, LA --\n"

 >> putStrLn "-- Some (mostly) fixed Discrete Models:"
 >> putStrLn("multi01 = " ++ show multi01
    ++ " -> " ++ show(map (pr multi01) [0, 1, 2] ) )
 >> putStrLn("pr coin01 H and T = " ++ show(map (pr coin01) [H, T] ) )
 >> putStrLn("pr coin02 H and T = " ++ show(map (pr coin02) [H, T] ) )
 >> putStrLn("pr twoCoin same   = " ++ show(map (pr twoCoin) [(H,H),(T,T)] ) )
 >> putStrLn ""

 >> putStrLn("msgBase  2 bitsMdl [0,1]= "
    ++ show( msgBase 2 bitsMdl [0,1] ) ++ " bits" )
 >> putStrLn("msg1Base 2 bitsMdl      = "
    ++ show( msg1Base 2 bitsMdl ) ++ " bits" )
 >> putStrLn("msg2Base 2 bitsMdl [0,1]= "
    ++ show( msg2Base 2 bitsMdl [0,1] ) ++ " bits" )
 >> putStrLn("msg1Base 2 (mixture bitsMdl ...) = "                    --mixture
    ++ show( msg1Base 2 (mixture (Mix bitsMdl [bitsMdl, bitsMdl])) )++" bits")
 >> putStrLn ""

 >> test_model wallaceIntModel [0..10]
 >> putStrLn ""

 >> test_model (geometric 3)   [0..10]  -- NB. (i) n>=0, & (ii) param=mean
 >> test_model (poisson   3)   [0..10]  -- NB. (i) n>=0, & (ii) param=mean
 >> test_model (intNormal 3 2) [0..10]
 >> putStrLn "c.f. N(,)"
 >> test_model (normal 1 3 2) (map fromIntegral [0..10])
 >> putStrLn ""

 >> putStrLn "-- Some learned Discrete Models:"
 >> test_est estGeom  ints1
 >> test_est estPois  ints1
 >> test_est estInorm ints1
 >> putStrLn "\n-- Some Continuous models..."

 >> putStrLn "-- Some fixed Continuous Models:"
 >> test_model n01   flts1                     --N(0,1)
 >> test_model (T.model 0 0.1 (1,1,6)) flts1   --t(1,1,6)

 >> putStrLn "-- Some learned Continuous Models:"
 >> test_est estNorm flts1
 >> test_est estT    flts1
 >> test_est estNorm flts2
 >> test_est estT    flts2
 >> test_est estNorm flts3
 >> test_est estT    flts3

 >> putStrLn "\n-- NB. cts data stated to +/-0.05 in most tests."
 >> putStrLn "-- done --"
-- -------------------------------------------LA--http://www.allisons.org/ll/--
