--Prototype source code for numerical scientific computing in Haskell, ghc.
--Here: Test-harness main program to test module Sci (Sci.hs).
--Also see Sci.hs (and T.hs).
--Compile tests thus: ghc --make Sci  (has run under ghc 6.4.1, Feb 2008).

module Main where

import IP.Sci           -- * defined locally

-- Terminology:
--   "vector" is a 1-D array,
--   "matrix" is a 2-D array, and
--   "array"  is the most general term.

import Array  --a standard library.
-- For Haskell's 'Array' library, see...
--   http://haskell.org/ghc/docs/latest/html/libraries/array/Data-Array.html

-- -----------------------------------1/2008--LA--http://www.allisons.org/ll/--

display name arr = name ++ " = " ++ show arr ++ "; bounds=" ++ show(bounds arr)

main =
 let a1 = listArray (0,6) [7,1,4,1,5,8,3]
     v1 = listArray (0,1) [1, 2]
     m1 = listArray ((0,0),(1,1)) [ 0, 1, 10, 11]    --2x2
     mf = listArray ((0,0),(1,1)) [ 0.0, 1.1, 10.1, 11.11]
     m2 = listArray ((0,0),(2,2)) [1..9]             --3x3
     v2 = toVector [1,2]
     v3 = toVector [1..3]
     m23 = listArray ((0,0),(1,2)) [1..6]  -- 2x3
     r23 = listArray ((0,0),(1,1)) [0.0, 1.1, 2.2, 3.3]
     f (x,y) = (x-1)**4 + 2*(y-2)**4       -- fn of 2 params, min @ (1,2)
     fa = f . arrayToT2                    -- equiv fn of an array

 in putStrLn "-- Tests of the proto Haskell Sci library, LA --\n"

 >> putStrLn( "integral -pi/2 pi/2 cos = "
	   ++ show( integrate (-pi/2) (pi/2) 1001 cos ) )
 >> putStrLn("delta F and D " ++ show ((delta 1)::Float, (delta 1)::Double) )

 >> (let sb1010 = stirlingBase 10 10
     in print("stirlingBase 10 10 = " ++ show sb1010
	   ++ ", 10**...="++show (10**sb1010) ))
 >> putStrLn("factorials " ++ show( map (exp.logFactorial) [1..10] ))
 >> putStrLn("log G's   "++show( map logGamma [0.01, 0.1, 1.0, 10.0, 100.0]))
 >> putStrLn("digammas  "++show( map digamma  [0.01, 0.1, 1.0, 10.0, 100.0]))
 >> putStrLn("  and at 1.0, adaptively, " ++ show(diffAdaptive logGamma 1.0))
 >> putStrLn("trigammas "++show( map trigamma [0.01, 0.1, 1.0, 10.0, 100.0]))
 >> putStrLn ""

 >> putStrLn( (display "a1" a1) ++ "\n  a1!2=" ++ show(a1!2) )
 >> putStrLn( display "toVector..." (toVector [10,20,30,40]) )
 >> putStrLn( display "m1" m1 )
 >> putStrLn( "|m2| = " ++ show(det m2) )
 >> putStrLn( display "row m1 1" (row m1 1) )
 >> putStrLn( display "col m1 1" (col m1 1) )
 >> putStrLn( display "mf*mf" (mmTimes mf mf) )
 >> putStrLn( display "v1" v1 )
 >> putStrLn( display "normaliseArray v1" (normaliseArray v1) )
 >> putStrLn( "dot v1 v1 = " ++ show(dot v1 v1) )
 >> putStrLn( display "arrayPlus v3 v3" (arrayPlus v3 v3) )
 >> putStrLn( display "arrayPlus r23 r23" (arrayPlus r23 r23) )
 >> putStrLn ""

 >> putStrLn( display "v2 " v2 )
 >> putStrLn( display "v3 " v3 )
 >> putStrLn( display "m23" m23 )
 >> putStrLn( display "mTranspose m23" (mTranspose m23) )
 >> putStrLn( display "vmTimes v2 m23" (vmTimes v2 m23) )
 >> putStrLn( display "mmTimes (vec2row v2) m23" (mmTimes (vec2row v2) m23) )
 >> putStrLn( display "mvTimes m23 v3" (mvTimes m23 v3) )
 >> putStrLn( display "mmTimes m23 (vec2col v3)" (mmTimes m23 (vec2col v3)) )
 >> putStrLn ""

 >> putStrLn( display "(trans m23)*m23" (mmTimes (mTranspose m23) m23) )
 >> putStrLn( display "m23*(trans m23)" (mmTimes m23 (mTranspose m23)) )
 >> putStrLn ""

 >> putStrLn( display "grad fa (2,3)  " (grad fa   (arrayFromT2 (2,3)) ) )
 >> putStrLn( display "gradAdaptive fa (2,3)  "
                         (gradAdaptive fa (arrayFromT2 (2,3)) ) )
 >> putStrLn( display "argmin1 (x!0 > 1.5) fa "
         (argmin1 (axisConstraint 0 [GT] 1.5) fa (grad fa) (arrayFromT2(2,3))))
 >> putStrLn( display "hessian fa (1,2)" (hessian fa (arrayFromT2 (1,2)) ) )

 >> putStrLn "\n-- done --"
-- ------------------------------------------------------------------------LA--
