-- See:
-- Lloyd ALLISON. Models for machine learning and data mining in
--   functional programming.       doi:10.1017/S0956796804005301
--   J. Functional Programming, 15(1), pp.15-32, Jan. 2005 (online 23 Jul 2004)
-- Lloyd ALLISON. Types and classes of machine learning and data mining.
--   ACSC2003, pp.207-215, February 2003.
-- Lloyd ALLISON. A programming paradigm for machine learning, with
--   a case study of Bayesian networks.  ACSC2006, January 2006.
--
-- Author: Lloyd ALLISON
-- This program is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License (GPL) as published by
-- the Free Software Foundation; either version 2 of the License, or (at
-- your option) any later version.  This program is distributed in the hope
-- that it will be useful, but without any warranty, without even the implied
-- warranty of merchantability or fitness for a particular purpose.  See the
-- GNU General Public License for more details.  You should have received a
-- copy of the GNU General Public License with this program; if not, write to:
-- Free Software Foundation, Inc., Boston, MA 02111, USA.

module IP.MixtureModels (module IP.MixtureModels) where

import IP.Utilities
import IP.Classes
import IP.Models

-- Estimate a mixture given a list of (component) estimators & a dataset,
-- expectation maximisation (EM) loop for a given number of components,
-- guess memberships -> mixture -> loop{ -> memberships -> mixture },
-- membership refers to a datum fractionally "belonging to" one or more
-- components of the mixture.
-- (This estimator is simple proof-of-concept code.)

estMixture ests dataSet =  -- [estimator] -> [dataSpace] -> model of dataSpace
 let                       -- i.e. [estimator] -> estimator

  memberships  (Mix mixer components)  =                 -- memberships|Mixture
    let doAll (d:ds) = prepend (doOne d) (doAll ds)      -- all data
        doAll  []    = map (\x -> []) components
        doOne  datum = normalise(                        -- one datum
          zipWith (\c -> \m -> (pr mixer c)*(pr m datum)) [0..] components)
                                  -- pr(c) * pr(datum|c)  for class #c = m
    in doAll dataSet

  randomMemberships =
    let doAll seed  []    = map (\_ -> []) ests
        doAll seed (_:ds) =                                         -- all data
          let doOne seed  []      ans = (seed, normalise ans)
              doOne seed (_:ests) ans =                            -- one datum
                doOne (prng seed) ests ((fromIntegral(1+ seed `mod` 10)) : ans)
          in let (seed2, forDatum) = doOne seed ests []
             in prepend forDatum (doAll seed2 ds)
    in doAll 4321 dataSet   -- should use a better prng!

  fit [] [] = []                                          -- Models|memberships
  fit (est:ests) (mem:mems) = (est dataSet mem) : (fit ests mems)

  fitMixture mems = Mix (freqs2model (map (foldl (+) 0) mems))  -- weights
                        (fit ests mems)                         -- components

  cycle    mx = fitMixture (memberships mx)                     -- EM step
  cycles 0 mx = mx
  cycles n mx = cycles (n-1) (cycle mx)                         -- n x cycle

 in mixture( cycles 50 (fitMixture randomMemberships) )   -- default 50

-- -------------------------------------------LA--http://www.allisons.org/ll/--
